@@ -3,16 +3,13 @@ Paco References
 """
 
 from paco.models import vocabulary, schemas
+from paco.models.locations import get_parent_by_interface
 from paco.models.exceptions import InvalidPacoReference
 from ruamel.yaml.compat import StringIO
-from zope.schema.interfaces import ITextLine
-from zope.interface import implementer, Interface
 from operator import itemgetter
 import importlib
-import os
 import pathlib
 import ruamel.yaml
-import zope.schema
 
 
 ami_cache = {}
@@ -27,7 +24,8 @@ class YAML(ruamel.yaml.YAML):
         if dumps:
             return stream.getvalue()
 
-
+# ToDo: this is duplicated between reftypes and references
+import zope.schema
 class InvalidPacoReferenceString(zope.schema.ValidationError):
     __doc__ = 'PacoReference must be of type (string)'
 
@@ -52,59 +50,6 @@ def is_ref(paco_ref, raise_enabled=False):
     if raise_enabled: raise InvalidPacoReferenceRefType
     return False
 
-class FileReference():
-    pass
-
-class StringFileReference(FileReference, zope.schema.Text):
-    """Path to a file on the filesystem"""
-
-    def constraint(self, value):
-        """
-        Validate that the path resolves to a file on the filesystem
-        """
-        return True
-        # ToDo: how to get the PACO_HOME and change to that directory from here?
-        #path = pathlib.Path(value)
-        #return path.exists()
-
-class YAMLFileReference(FileReference, zope.schema.Object):
-    """Path to a YAML file"""
-
-    def __init__(self, **kw):
-        self.schema = Interface
-        self.validate_invariants = kw.pop('validate_invariants', True)
-        super(zope.schema.Object, self).__init__(**kw)
-
-class PacoReference(zope.schema.Text):
-
-    def __init__(self, *args, **kwargs):
-        self.str_ok = False
-        self.schema_constraint = ''
-        if 'str_ok' in kwargs.keys():
-            self.str_ok = kwargs['str_ok']
-            del kwargs['str_ok']
-        # schema_constraint is a string name of an ISchema
-        # if a Schema is passed, it is converted to a string
-        if 'schema_constraint' in kwargs.keys():
-            self.schema_constraint = kwargs['schema_constraint']
-            if hasattr(self.schema_constraint, '__name__'):
-                self.schema_constraint = self.schema_constraint.__name__
-            del kwargs['schema_constraint']
-        super().__init__(*args, **kwargs)
-
-    def constraint(self, value):
-        """
-        Limit text to the format 'word.ref chars_here.more-chars.finalchars100'
-        """
-        if self.str_ok and is_ref(value) == False:
-            if isinstance(value, str) == False:
-                raise InvalidPacoReferenceString
-                #return False
-            return True
-
-        return is_ref(value, raise_enabled=True)
-
-
 class Reference():
     """
     Reference to something in the paco.models
@@ -127,15 +72,24 @@ class Reference():
         self.resource_ref = None
         self.region = None
 
-        if self.type == 'netenv':
+        if self.type == 'netenv' or self.type == 'service':
+            # paco.ref service.describe.<account>.<region>.myapp
+            # pace.ref netenv.dev.<account>.<region>.applications
             # do not try to find region for short environment refs like 'paco.ref netenv.mynet.prod'
             if len(self.parts) > 3:
                 if self.parts[3] in vocabulary.aws_regions.keys():
                     self.region = self.parts[3]
-
         if is_ref(self.raw) == False:
             print("Invalid Paco reference: %s" % (value))
-            #raise StackException(PacoErrorCode.Unknown)
+
+    def get_account(self, project, resource):
+        "Account object this reference belongs to"
+        if self.type == 'service':
+            return project.accounts[self.parts[2]]
+        elif self.type == 'netenv':
+            env_reg = get_parent_by_interface(resource, schemas.IEnvironmentRegion)
+            return get_model_obj_from_ref(env_reg.network.aws_account, project)
+        return None
 
     @property
     def last_part(self):
@@ -179,6 +133,57 @@ class Reference():
             ref=self
         )
 
+    def get_model_obj(self, project):
+        return get_model_obj_from_ref(self, project)
+
+    def secret_base_ref(self):
+        """
+Secret Manager Secrets can be a ref to the Secret:
+
+  paco.ref netenv.mynet.dev.eu-central-1.secrets_manager.myapp.mygroup.mydb
+
+Or they can be a ref to a JSON field within the Secret value:
+
+  paco.ref netenv.mynet.dev.eu-central-1.secrets_manager.myapp.mygroup.mydb.myjsonfield
+
+Either ref type can also specify the ARN of the Secret:
+
+  paco.ref netenv.mynet.dev.eu-central-1.secrets_manager.myapp.mygroup.mydb.arn
+  paco.ref netenv.mynet.dev.eu-central-1.secrets_manager.myapp.mygroup.mydb.myjsonfield.arn
+
+Return only the base ref and strip the final part that specifies the JSON Field from the ref.
+For all of the above examples, this would return:
+
+  paco.ref netenv.mynet.secrets_manager.myapp.mygroup.mydb
+
+        """
+        new_ref_obj = self
+        if self.type == 'netenv':
+            # remove final .arn but allow for a secret named 'arn'
+            #   paco.ref netenv.mynet.dev.eu-central-1.secrets_manager.myapp.mygroup.arn
+            #   paco.ref netenv.mynet.dev.eu-central-1.secrets_manager.myapp.mygroup.arn.arn
+            #   paco.ref netenv.mynet.dev.eu-central-1.secrets_manager.myapp.mygroup.arn.myjson
+            #   paco.ref netenv.mynet.dev.eu-central-1.secrets_manager.myapp.mygroup.arn.myjson.arn
+            if len(self.parts) == 7:
+                pass
+            elif len(self.parts) > 7:
+                new_ref = self.parts[:8]
+                new_ref = '.'.join(new_ref)
+                new_ref_obj = Reference(f'paco.ref {new_ref}')
+            else:
+                raise InvalidPacoReference("Not a valid Secret ref")
+        elif self.type == 'service':
+            # service.notification.secrets_manager.myapp.mygroup.mysecret
+            if len(self.parts) > 5:
+                new_ref = self.parts[:6]
+                new_ref = '.'.join(new_ref)
+                new_ref_obj = Reference(f'paco.ref {new_ref}')
+            else:
+                raise InvalidPacoReference("Not a valid Secret ref")
+        else:
+            raise NotImplemented('What kind of Secret do you have?!?')
+        return new_ref_obj
+
 def get_model_obj_from_ref(ref, project, referring_obj=None):
     """Resolves the reference to the model object it refers to.
     ref can be either a string or a Reference object.
@@ -198,6 +203,8 @@ def get_model_obj_from_ref(ref, project, referring_obj=None):
             obj = next_obj
         else:
             message = f"\nCould not find model at {ref.raw}\n"
+            message = f"\nLook-up reached: {obj.paco_ref_parts}\n"
+            message = f"\nCould not find next part: {ref.parts[part_idx]}\n"
             if referring_obj != None:
                 message += f"Object: {referring_obj.paco_ref_parts}\n"
             if ref.parts[0] in ['iam', 'codecommit', 'ec2', 'snstopics', 's3', 'route53', 'resources']:
@@ -269,8 +276,7 @@ def raise_invalid_reference(ref, obj, name):
     the name that failed the next look-up.
     """
     raise InvalidPacoReference("""
-Invalid Paco reference:
-{}
+Reference: {}
 
 Reference look-up failed at '{}' trying to find name '{}'.
 """.format(ref.raw, obj.paco_ref_parts, name))
