@@ -3,11 +3,9 @@ import paco.models.networks
 import paco.models.loader
 import os
 import troposphere
-import inspect
 import unittest
 from paco.models import load_project_from_yaml, schemas
 from paco.models.project import Project
-from paco.models.networks import SecurityGroup
 
 def fixtures_path():
     # find the project root directory
@@ -38,6 +36,11 @@ class BaseTestModelLoader(unittest.TestCase):
 class Testpacodemo(BaseTestModelLoader):
 
     project_name = 'pacodemo'
+
+    def setUp(self):
+        self.demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
+        self.demo_app = self.demo_env['applications']['app']
+
     def test_project(self):
         assert isinstance(self.project, Project)
         assert self.project.name == 'waterbear-networks'
@@ -80,6 +83,23 @@ class Testpacodemo(BaseTestModelLoader):
         assert demo_env['default'].network.vpc.segments['webapp'].internet_access == False
         assert demo_env['default'].network.vpc.segments['public'].internet_access == True
 
+    def test_lbapplication(self):
+        demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
+        alb = demo_env['applications']['app'].groups['site'].resources['alb']
+        assert schemas.ILoadBalancer.providedBy(alb)
+        assert schemas.IListener.providedBy(alb.listeners['https'])
+        assert alb.listeners['https'].rules['app_forward'].name == 'app_forward'
+        assert alb.listeners['https'].rules['app_forward'].rule_type == 'forward'
+
+    def test_lbnetwork(self):
+        demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
+        nlb = demo_env['applications']['app'].groups['site'].resources['nlb']
+        assert schemas.INetworkLoadBalancer.providedBy(nlb)
+        # assert schemas.IListener.providedBy(alb.listeners['https'])
+        # assert alb.listeners['https'].rules['app_forward'].name == 'app_forward'
+        # assert alb.listeners['https'].rules['app_forward'].rule_type == 'forward'
+
+
     def test_asg(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
         asg = demo_env.applications['app'].groups['site'].resources['webapp']
@@ -213,6 +233,18 @@ class Testpacodemo(BaseTestModelLoader):
         assert schemas.IDBParameterGroup.providedBy(dbparams)
         assert dbparams.parameters['block_encryption_mode'] == 'aes-128-ecb'
 
+    def test_iamuser_resource(self):
+        demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
+        github_user = demo_env['applications']['app'].groups['cicd'].resources['github_user']
+        assert schemas.IIAMUserResource.providedBy(github_user)
+        assert github_user.allows[0] == 'paco.ref netenv.pacodemo.demo.us-west-2.app.app.groups.cicd.resources.ecrrepo'
+
+    def test_pinpointapp(self):
+        pinpointapp = self.demo_app.groups['market'].resources['pinpointapp']
+        assert schemas.IPinpointApplication(pinpointapp)
+        assert pinpointapp.email_channel.from_address == 'bob@example.com'
+        assert pinpointapp.sms_channel.sender_id == 'MisterId'
+
     def test_cloudwatch_logging(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
         demo_webapp = demo_env['applications']['app'].groups['site'].resources['webapp']
@@ -335,13 +367,35 @@ class Testpacodemo(BaseTestModelLoader):
         cluster = demo_env['applications']['app'].groups['container'].resources['ecs_cluster']
         assert schemas.IECSCluster.providedBy(cluster)
         ecs_service_config = demo_env['applications']['app'].groups['container'].resources['ecs_services']
+
+        assert ecs_service_config.setting_groups['app_pool'].secrets[0].name, 'TWO_SECRET'
+        assert ecs_service_config.setting_groups['app_pool'].environment[1].value, 'down'
+
+        simple_app_service = ecs_service_config.services['simple_app']
         assert ecs_service_config.services['simple_app'].desired_count, 2
         assert ecs_service_config.services['simple_app'].load_balancers[0].container_name, 'hello'
-        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].cpu, 10
-        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].depends_on[0].condition, 'START'
-        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].health_check.retries, 5
-        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].ulimits[0].hard_limit, 1000
-        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].user, 'www-data'
+        hello_def = ecs_service_config.task_definitions['hello_web'].container_definitions['hello']
+        assert hello_def.cpu, 10
+        assert hello_def.depends_on[0].condition, 'START'
+        assert hello_def.health_check.retries, 5
+        assert hello_def.ulimits[0].hard_limit, 1000
+        assert hello_def.user, 'www-data'
+        assert hello_def.secrets[0].value_from, 'paco.ref netenv.pacodemo.secrets_manager.one.two.another'
+        assert hello_def.environment[0].name, 'hello'
+
+        assert simple_app_service.target_tracking_scaling_policies['memory'].scale_in_cooldown, 300
+        assert simple_app_service.target_tracking_scaling_policies['cpu'].predefined_metric, 'ECSServiceAverageCPUUtilization'
+        assert simple_app_service.target_tracking_scaling_policies['cpu'].target, 70
+
+    def test_cognito(self):
+        auth_app = self.demo_app.groups['auth']
+        userpool = auth_app.resources['userpool']
+        assert schemas.ICognitoUserPool(userpool)
+        assert userpool.app_clients['bob'].generate_secret, True
+        assert userpool.user_creation.invite_message_templates.email_message, "Would you like to test the email?"
+        assert userpool.lambda_triggers.pre_sign_up.startswith('paco.ref ')
+        idp = auth_app.resources['ident']
+        assert schemas.ICognitoIdentityPool(idp)
 
     def test_rds(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
@@ -387,8 +441,12 @@ class Testpacodemo(BaseTestModelLoader):
 
     def test_api_gateway_rest_api(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
-        api_gra = demo_env['applications']['app'].groups['restapi'].resources['api_gateway_rest_api']
-        assert len(api_gra.body_file_location) > 1
+        api = demo_env['applications']['app'].groups['restapi'].resources['api_gateway_rest_api']
+        assert len(api.body_file_location) > 1
+        api.resources['idle'].path_part, '/idle'
+        api.resources['idle'].child_resources['view'].path_part, '{view}'
+        api.resources['idle'].child_resources['view'].child_resources['one'].path_part, 'one'
+        api.resources['idle'].child_resources['view'].child_resources['two'].path_part, 'two'
 
     def test_health_checks(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
@@ -421,6 +479,9 @@ class Testpacodemo(BaseTestModelLoader):
         assert myvault.title == "All data for MyApp (myapp) application"
         assert myvault.plans['ebs_daily'].plan_rules[0].schedule_expression == 'cron(0 7 ? * * *)'
 
+        assert myvault.plans['ebs_daily'].plan_rules[0].copy_actions[0].destination_vault.startswith('arn:aws:backup')
+        assert myvault.plans['ebs_daily'].plan_rules[0].copy_actions[1].lifecycle_delete_after_days == 100
+
     # dashboard_file is not reading correctly? disable for now
     def test_dashboard(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
@@ -429,6 +490,20 @@ class Testpacodemo(BaseTestModelLoader):
         assert dashboard.title == "Demo-Dashboard"
         assert 'WebAsg.name' in dashboard.variables
 
+    def test_dynamodb(self):
+        dynamodb = self.demo_app.groups['storage'].resources['dynamodb']
+        assert schemas.IDynamoDB.providedBy(dynamodb)
+        assert dynamodb.default_provisioned_throughput.read_capacity_units == 5
+        concert = dynamodb.tables['concert']
+        assert concert.attribute_definitions[0].name == 'ArtistId'
+        assert concert.key_schema[0].type == 'HASH'
+        assert concert.global_secondary_indexes[0].index_name == 'GSI'
+        assert concert.global_secondary_indexes[0].projection.type == 'KEYS_ONLY'
+        assert concert.provisioned_throughput.write_capacity_units == 10
+        assert concert.target_tracking_scaling_policy.min_capacity == 5
+        discography = dynamodb.tables['discography']
+        assert discography.attribute_definitions[1].type == 'S'
+
     def test_iotcore(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
         iottopic = demo_env['applications']['app'].groups['iot'].resources['iottopic']
